package com.codecaballero.proyectofinal;

import android.content.Context;

import org.opencv.core.Point;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

/**
 * Created by android on 22/07/15.
 */
public class ArchivoXML
{

    static final String FECHA="fecha";
    static final String ASIGNATURA= "asignatura";
    static final String EXAMEN = "examen";
    static final String PUNTO = "punto";
    static final String TAMANO = "tamano";
    static final String X = "x";
    static final String Y = "y";
    static final String MARCADO = "marcado";




    String xmlFile;

    private List<MiCasilla> modeloExamen = new ArrayList<>();
    float distancia = 10;

    /**
     * Convierte la lista modeloExamen en un archivo XML
     * @param context
     * @param modeloExamen tiene que ser del tipo ArrayList<MiCasilla>
     * @return true si la operación se ha realizaco correctamente
     */
    public static boolean crearXML(Context context, ArrayList<MiCasilla> modeloExamen){
        String xmlString;
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();//Documetno que se va a crear

            Element rootElement = document.createElement(EXAMEN); //Etiqueta catalogo
           /* rootElement.setAttribute(ASIGNATURA, "asignatura"); //atributo asignatura
            rootElement.setAttribute(FECHA, "1/1/1980"); //atributo fecha*/
            //rootElement.setAttribute(TAMANO,String.valueOf(modeloExamen.get(1).getTamanoCasilla()));
            document.appendChild(rootElement); //añado la rama al documento

            for (int i = 0; i<modeloExamen.size();i++) {
                Point P1 = modeloExamen.get(i).getCentro();
                Boolean marcado = modeloExamen.get(i).isEstaMarcado();


                Element puntoElement = document.createElement(PUNTO);//Creo la nueva etiqueta punto
                puntoElement.setAttribute("id","punto"+i);//TODO ??
                rootElement.appendChild(puntoElement); //añado puntoelement a examen element

                Element xElement = document.createElement(X);
                puntoElement.appendChild(xElement);
                xElement.appendChild(
                        document.createTextNode(String.valueOf(P1.x)));//Creo texto  de la etiqueta

                Element yElement = document.createElement(Y);
                puntoElement.appendChild(yElement);
                yElement.appendChild(document
                        .createTextNode(String.valueOf(P1.y)));

                //TODO
                Element marcadoElement = document.createElement(MARCADO);
                puntoElement.appendChild(marcadoElement);
                marcadoElement.appendChild(document
                        .createTextNode(String.valueOf(marcado)));
            }

            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            Properties outFormat = new Properties();
            outFormat.setProperty(OutputKeys.INDENT, "yes");
            outFormat.setProperty(OutputKeys.METHOD, "xml");
            outFormat.setProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
            outFormat.setProperty(OutputKeys.VERSION, "1.0");
            outFormat.setProperty(OutputKeys.ENCODING, "UTF-8");
            transformer.setOutputProperties(outFormat);
            DOMSource domSource =
                    new DOMSource(document.getDocumentElement());
            OutputStream output = new ByteArrayOutputStream();
            StreamResult result = new StreamResult(output);
            transformer.transform(domSource, result);

            xmlString = output.toString();



        } catch (ParserConfigurationException e) {
            e.printStackTrace();
            return false;
        } catch (TransformerConfigurationException e) {
            e.printStackTrace();
            return false;
        } catch (TransformerException e) {
            e.printStackTrace();
            return false;
        }

        if (writeToFile(context,xmlString)){ //Escribo _todo en el archivo
            return true;
        }else{
            return false;
        }

    }

    /**
     * Escribe un String en un Archivo
     *
     * Se almacena en sdcard/data/data/code/caballero/ProyectoFinal/examenXML
     *
     * @param context
     * @param data El string al escribir en archivo
     * @return true si las operaciones se han realizado correctamente
     */
    private static boolean writeToFile(Context context, String data) {
        String path = context.getExternalFilesDir(null).getAbsolutePath();
        File file = new File(path + "/examen.xml");
        try {
            FileOutputStream stream = new FileOutputStream(file);
            stream.write(data.getBytes());
            stream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return true;
    }

    /**
     * Obtiene de un XML las coordenadas de las casillas y si estaban marcadas
     * @param context
     * @return el Array con las coordenadas y si estan marcadas
     */

    public static ArrayList<PointBool> getXYFromXml(Context context){
        ArrayList<PointBool> arrayList = new ArrayList<>();
        String path = context.getExternalFilesDir(null).getAbsolutePath();
        try {
            //FileInputStream file = context.openFileInput(path+"/examen.xml");
            FileInputStream file = new FileInputStream(new File(path+"/examen.xml"));
            //FileInputStream file = new FileInputStream(new File(path+"/books.xml"));

            //DOM
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

            DocumentBuilder builder = factory.newDocumentBuilder();
            Document dom = builder.parse(file);

            //dom.getDocumentElement().normalize();//TODO error

            Element root = dom.getDocumentElement();


            NodeList items = root.getElementsByTagName(PUNTO);
            //NodeList items = root.getElementsByTagName("person");

            for (int i =0;i<items.getLength();i++){
                Node punto = items.item(i);
                if (punto.getNodeType()==Node.ELEMENT_NODE)
                {
                    Element puntoElemento = (Element) punto;
                    //punto X
                    NodeList puntoX = puntoElemento.getElementsByTagName(X);
                    Element puntoXelemento = (Element)puntoX.item(0);
                    NodeList textoPuntoX = puntoXelemento.getChildNodes();
                    Double cordX = Double.parseDouble(textoPuntoX.item(0).getNodeValue().trim());

                    //arrayList.add(new MiCasilla());

                    //punto y
                    NodeList puntoY = puntoElemento.getElementsByTagName(Y);
                    Element puntoYelemento = (Element)puntoY.item(0);
                    NodeList textoPuntoY = puntoYelemento.getChildNodes();
                    Double cordY = Double.parseDouble(textoPuntoY.item(0).getNodeValue().trim());

                    //estaMarcado TODO
                    NodeList estaMarcado = puntoElemento.getElementsByTagName(MARCADO);
                    Element estaMarcadoelemento = (Element)estaMarcado.item(0);
                    NodeList textoEstaMarcado = estaMarcadoelemento.getChildNodes();
                    Boolean estaMarca = Boolean.parseBoolean(textoEstaMarcado.item(0).getNodeValue().trim());

                    //Añadir
                    arrayList.add(i,new PointBool(new Point(cordX,cordY),estaMarca));
                }

            }

            file.close();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }


        return arrayList;
    }



}
