package com.codecaballero.proyectofinal;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.Toast;

import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Size;

import java.util.ArrayList;
import java.util.List;

/**
 * Actividad donde el usuario puede comprobar si las casillas estan marcadas
 */
public class CamaraActivity extends Activity implements CameraBridgeViewBase.CvCameraViewListener2{

    private static final String TAG = "ModeloActivity";
    private CameraBridgeViewBase cameraView;
    private ProcesadorCamara procesador;
    private int cam_anchura =640;// resolucion deseada de la imagen
    private int cam_altura=480;
    private List<Integer> correctos = new ArrayList<>();
    int numExa;
    Button bt;
    private boolean pararCamara;
    Button bt2;
    private BufferCasillas buffer;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        buffer = BufferCasillas.getInstancia();

        if (buffer.getBufferCasillasParaCorregir().isEmpty())
        {
            Toast.makeText(CamaraActivity.this,"No hay exámenes cargados",Toast.LENGTH_SHORT).show();
            finish();
        }else
        {
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

            Bundle extras = getIntent().getExtras();
            if (extras != null) {
                numExa = extras.getInt("OPCION");
            }
            setContentView(R.layout.activity_examen_corregir);
            bt = (Button) findViewById(R.id.btCorregir);
            bt2 = (Button) findViewById(R.id.btVista);


            getModeloDesdeXML(CamaraActivity.this);
            cameraView = (CameraBridgeViewBase) findViewById(R.id.vista_camara);
            cameraView.setCvCameraViewListener(this);
        }

    }
    private BaseLoaderCallback loaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            switch (status) {
                case LoaderCallbackInterface.SUCCESS:
                    Log.i(TAG, "OpenCV se cargo correctamente");
                    cameraView.setMaxFrameSize(cam_anchura, cam_altura);
                    cameraView.enableView();
                    break;
                default:
                    super.onManagerConnected(status);
                    break;
            }
        }
    };

    /**
     *
     * @param width -  the width of the frames that will be delivered
     * @param height - the height of the frames that will be delivered
     */
    @Override
    public void onCameraViewStarted(int width, int height) {
        procesador = new ProcesadorCamara();
        //procesador = new Procesador();
    }

    @Override
    public void onCameraViewStopped() {

    }

    @Override
    public Mat onCameraFrame(CameraBridgeViewBase.CvCameraViewFrame inputFrame) {
        Mat entrada;
        Mat salida = new Mat();
        //Mat entradaGris = inputFrame.gray();//entrada en gris
        if (!pararCamara){
            entrada = inputFrame.rgba();
            salida = entrada;
        } else {
            entrada = inputFrame.gray();
            salida = procesador.procesa(entrada);
        }


        bt.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pararCamara = true;
            }
        });

        bt2.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pararCamara = false;
                //cameraView.disableView();
            }
        });


        return salida;
    }



    @Override
    public void onPause() {
        super.onPause();
        if (cameraView != null)
            cameraView.disableView();
    }

    @Override
    public void onResume() {
        super.onResume();
        OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_2_4_9, this, loaderCallback);
    }
    @Override
    public void onDestroy() {
        super.onDestroy();
        if (cameraView != null)
            cameraView.disableView();
    }

    /**
     * Modifica el Singleton BufferCasillas con los datos del XML     *
     * Usar si se quiere obtener el modelo desde el XML
     * Obtiene desde el ArchivoXML los puntos marcados
     * @param context
     */
    private void getModeloDesdeXML(Context context){
        ArrayList<MiCasilla> array = new ArrayList<>();
        ArrayList<PointBool> pointBool = ArchivoXML.getXYFromXml(context);
        for (int i = 0;i<buffer.getBufferCasillas().size();i++){
            array.add(i,new MiCasilla(new Rect(new Point(0,0),new Size(0,0)), pointBool.get(i).isEstaMarcado()));
        }
        buffer.setBufferCasillas(array);
    }
}
