package com.codecaballero.proyectofinal;

import android.util.Log;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase para procesar la imagen de entrada de la plantilla para corregirla
 */
public class ProcesadorCamara
{

    private BufferCasillas buffer;


    private static int THINKNESS = 2;
    Mat paso_bajo;
    private List<MatOfPoint> contornoList;
    private Mat hierarchyMat = new Mat();
    MatOfPoint2f approx = new MatOfPoint2f();
    private boolean cuadradoEstaMarcado;
    float[] hist;
    Mat salidaPrueba = new Mat();

    public ProcesadorCamara() { //Constructor
        paso_bajo = new Mat();
        hist  = new float[256];
        buffer = BufferCasillas.getInstancia();

    }

    Mat entradaLimpia = new Mat();
    /**
     * Procesa la imagen buscando las casillas e identificando si están marcadas
     *
     * @param entrada: Entrada de la imagen por camara
     * @return: Devuelve la imagen con las casillas que están correctas (blanco) y las casillas incorrectas (negro)
     */
    public Mat procesa(Mat entrada) {
        buffer.clearCasillas();
        entradaLimpia = entrada.clone();
        Mat salida = new Mat();
        Mat salidaGray = new Mat();

        int filter_size = 17;
        Size s = new Size(filter_size, filter_size);
        Imgproc.blur(entrada, paso_bajo, s);

        //Hacer la resta. Los valores saturan a cero
        Core.subtract(paso_bajo, entrada, salida);

        //Aplicar ganancia para ver mejor
        Scalar ganancia = new Scalar(4);
        Core.multiply(salida, ganancia, salida);

        binarizacionOtsu(salida);

        salidaPrueba = salida.clone();

        //Dilatacion
       /* double tam = 3;
        Mat SE = Imgproc.getStructuringElement(Imgproc.MORPH_RECT,new Size(tam,tam));
        Imgproc.dilate(salida,salida,SE);*/
        //Imgproc.dilate(salidaPrueba,salidaPrueba,SE);

        findContous(salida, entrada);
        return entrada;
    }


    /**
     * Binariza la imagen utilizando el método Otsu
     *
     * @param image: Imagen a binarizar
     */
    public void binarizacionOtsu(Mat image) {
        Imgproc.threshold(image, image, 0, 255, Imgproc.THRESH_OTSU);

    }

    /**
     * Encuentra los contornos de las casillas y las corrige.
     *
     * Las casillas tienen que tener un tamaño entre 30 - 15 pixeles para descartar falsos positivos
     * Las casillas deben tener una altura y anchura similares
     *
     * Dibuja un recuadro blanco sobre las casillas correctas.
     * Dibuja un recuadro negro sobre las casillas incorrectas.
     *
     * @param image Imagen de entrada, debe estar binarizada.
     * @param salida Imagen de salida, con las casillas corregidas.
     */
    private void findContous(Mat image, Mat salida) {

        contornoList  = new ArrayList<MatOfPoint>();

        //Todo probar con los internos
        Imgproc.findContours(image, contornoList, hierarchyMat, Imgproc.RETR_EXTERNAL, Imgproc.CHAIN_APPROX_NONE);
        //Recorro cada contorno y veo si cumple una serie de condiciones
        Log.d("Tamaño lista contornos", String.valueOf(contornoList.size()));

        //if (contornoList.size() < 100)
        {
            for (int c = 0; c < contornoList.size(); c++)
            {
                //Devuelve la altura y anchura del contorno
                Rect contorno = Imgproc.boundingRect(contornoList.get(c));

                //Si el tamaño es menor de 10px se descarta
                if ((contorno.height <15)||(contorno.width <15))
                    continue;
                //TODO Si el tamaño es mayor de 30px se descarta
                if ((contorno.height >30)||(contorno.width >30))
                    continue;

                float ratio = (float)contorno.height/(float)contorno.width;
                //Asi compruebo si es un cuadrado, si la altura y la anchura son similares
                if (!(ratio>0.75 && ratio<1.25))
                    continue;

                double precisionAproximacion = Imgproc.arcLength(new MatOfPoint2f(contornoList.get(c).toArray()), true) * 0.03;
                Imgproc.approxPolyDP(new MatOfPoint2f(contornoList.get(c).toArray()), approx, precisionAproximacion , true);
                //Compruebo si tiene cuatro vertices el contorno
                //Las coordenadas de los vertices se almacenan en approx
                if (approx.size().height != 4.0)
                    continue; //Si no los tiene -> for

                final Point P3 = new Point(contorno.x, contorno.y);
                final Point P4 = new Point(contorno.x + contorno.width, contorno.y + contorno.height);



                cuadradoEstaMarcado = estaMarcado(entradaLimpia,contorno);
                Core.rectangle(salida, P3, P4, new Scalar(255, 255, 255), THINKNESS);

                //Añado la casilla en orden
                buffer.addOrdenCasillaParaCorregir(new MiCasilla(contorno,cuadradoEstaMarcado));
            }
            //Si tengo el mismo número de casillas corregidas que para corregir entonces empiezo a corregir
            if (buffer.getBufferCasillasParaCorregir().size() == buffer.getBufferCasillas().size()){
                //TODO

                for (int j = 0; j < buffer.getBufferCasillasParaCorregir().size();j++){

                    if (buffer.getBufferCasillasParaCorregir().get(j).isEstaMarcado() && !buffer.getBufferCasillas().get(j).isEstaMarcado()){

                        Point P1 = buffer.getBufferCasillasParaCorregir().get(j).getCuadrado().tl();
                        Point P2 = buffer.getBufferCasillasParaCorregir().get(j).getCuadrado().br();
                        Core.rectangle(salida, P1, P2, new Scalar(0, 0, 255), THINKNESS);

                    }

                    if (!buffer.getBufferCasillasParaCorregir().get(j).isEstaMarcado() && buffer.getBufferCasillas().get(j).isEstaMarcado()){

                        Point P1 = buffer.getBufferCasillasParaCorregir().get(j).getCuadrado().tl();
                        Point P2 = buffer.getBufferCasillasParaCorregir().get(j).getCuadrado().br();
                        Core.rectangle(salida, P1, P2, new Scalar(0, 0, 255), THINKNESS);

                    }
                }
            }
        }

    }


    /**
     * Comprueba si una casilla esta marcada
     *
     * A partir del recorte de la imagen y la imagen principal se genera una subimagen que se analiza
     * y se comprueba si esta marcada.
     *
     * @param entrada1 imagen principal
     * @param rect recorte donde esta la casilla
     * @return true si la casilla esta marcada
     */
    public boolean estaMarcado(Mat entrada1, Rect rect){

        int factCor = (int) (rect.size().height *0.20);//Factor de correcion para crear el cuadrado interno
        Rect rectCorregida;
        Point p1 = new Point(rect.x + factCor, rect.y + factCor);
        Point p2 = new Point(rect.width - factCor,rect.height - factCor);
        if (rect.width >0) {
            rectCorregida = new Rect(rect.x + factCor, rect.y + factCor,rect.width - factCor,rect.height - factCor);
            //rectCorregida = new Rect(p1,p2);
        }else{
            rectCorregida = rect;
        }

        //TODO
        Mat recorte = entrada1.submat(rectCorregida);

       /* Log.e("entrada1","\n" +entrada1.dump());

        Log.e("Recorte","\n" +recorte.dump());*/
        List<MatOfPoint> contorno = new ArrayList<MatOfPoint>();
        Mat hierarchy = new Mat();
        binarizacionOtsu(recorte);
        Imgproc.findContours(recorte, contorno, hierarchy, Imgproc.RETR_LIST, Imgproc.CHAIN_APPROX_NONE);

        return contorno.size() >= 4;

    }

}
