package com.codecaballero.proyectofinal;

import android.util.Log;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase que sirve para procesar la imagen que sera modelo para corregir el examen/test
 */
public class ProcesadorModelo extends ProcesadorCamara
{

    //Buffer que guarda todas las casillas. Este array se limpia cada vez que se procesa una imagen nueva.
    //public ArrayList<MiCasilla> bufferCasillas = new ArrayList<>();

    private BufferCasillas buffer;

    private static int THINKNESS = 2;

    Mat paso_bajo;
    private List<MatOfPoint> contornoList;
    private Mat hierarchyMat = new Mat();

    MatOfPoint2f approx = new MatOfPoint2f();
    private boolean cuadradoEstaMarcado;
    float[] hist;

    private boolean botonHaSidoPulsado = false;

    public ProcesadorModelo() { //Constructor
        paso_bajo = new Mat();
        hist  = new float[256];
        buffer = BufferCasillas.getInstancia();

    }

    Mat entradaLimpia = new Mat();


    public Mat procesa(Mat entrada) {
        //TODO Limpio el Buffer
        buffer.clears();
        entradaLimpia = entrada.clone();
        Mat salida = new Mat();
        Mat salidaGray = new Mat();

        int filter_size = 30;
        Size s = new Size(filter_size, filter_size);
        Imgproc.blur(entrada, paso_bajo, s);

        //Hacer la resta. Los valores saturan a cero
        Core.subtract(paso_bajo, entrada, salida);

        //Aplicar ganancia para ver mejor
        Scalar ganancia = new Scalar(4);
      //  Core.multiply(salida, ganancia, salida);

        //Imgproc.cvtColor(salida, salidaGray, Imgproc.COLOR_RGB2GRAY);

        binarizacionOtsu(salida);


        //Dilatacion
       /* double tam = 2;
        Mat SE = Imgproc.getStructuringElement(Imgproc.MORPH_RECT,new Size(tam,tam));
        Imgproc.dilate(salida, salida, SE);*/
        //Imgproc.dilate(salidaPrueba,salidaPrueba,SE);

        findContous(salida, entrada);

        return entrada;
    }

    private void findContous(Mat image, Mat salida) {
        Mat entradaFind = image.clone();
        contornoList  = new ArrayList<MatOfPoint>();

        //Todo probar con los internos
        Imgproc.findContours(image, contornoList, hierarchyMat, Imgproc.RETR_EXTERNAL, Imgproc.CHAIN_APPROX_NONE);
        //Recorro cada contorno y veo si cumple una serie de condiciones
        Log.d("Tamaño lista contornos", String.valueOf(contornoList.size()));

        //if (contornoList.size() < 100)
        {
            for (int c = 0; c < contornoList.size(); c++)
            {
                //Devuelve la altura y anchura del contorno
                Rect contorno = Imgproc.boundingRect(contornoList.get(c));

                //Si el tamaño es menor de 10px se descarta
                if ((contorno.height <15)||(contorno.width <15))
                    continue;
                //TODO Si el tamaño es mayor de 30px se descarta
                if ((contorno.height >30)||(contorno.width >30))
                    continue;

                float ratio = (float)contorno.height/(float)contorno.width;
                //Asi compruebo si es un cuadrado, si la altura y la anchura son similares
                if (!(ratio>0.75 && ratio<1.25))
                    continue;

                //TODO Aprox

                Imgproc.approxPolyDP(new MatOfPoint2f(contornoList.get(c).toArray()), approx, Imgproc.arcLength(new MatOfPoint2f(contornoList.get(c).toArray()), true) * 0.03, true);
                //Compruebo si tiene cuatro vertices el contorno
                if (approx.size().height != 4.0)
                    continue; //Si no los tiene -> for

                final Point P3 = new Point(contorno.x, contorno.y);
                final Point P4 = new Point(contorno.x + contorno.width, contorno.y + contorno.height);



               cuadradoEstaMarcado = estaMarcado(entradaLimpia,contorno);
                //Si esta marcado con una X pintamos el rectangulo Azul

                if (cuadradoEstaMarcado) {//green
                    //Core.rectangle(entrada, P3, P4, new Scalar(0, 0, 255), 2);
                    Core.rectangle(salida, P3, P4, new Scalar(0, 0, 255), THINKNESS);
                } else {
                    //Core.rectangle(entrada, P3, P4, new Scalar(255, 0,0),2);
                    Core.rectangle(salida, P3, P4, new Scalar(255, 0, 0), THINKNESS);
                }
                //TODO mod 17:09
            /*if (!estaEnLaListaCuadrados(contorno,singleton.getMiListaCuadradosModelo()))
            {
                MiCasilla miCasilla = new MiCasilla(contorno,cuadradoEstaMarcado);
                singleton.addOrdenMiCuadradoModelo(miCasilla);
            }*/

                buffer.addOrdenMiCuadradoModelo(new MiCasilla(contorno,cuadradoEstaMarcado));
            }

        }

    }


}
